@@ -25,6 +25,7 @@ class Preferences(QDialog):
         self.origInterfaceLang = self.config['interfaceLang']
         self.dialog = ankiqt.forms.preferences.Ui_Preferences()
         self.dialog.setupUi(self)
+        self.needDeckClose = False
         self.supportedLanguages = [
             (u"Bahasa Melayu", "ms"),
             (u"Dansk", "da"),
@@ -35,6 +36,7 @@ class Preferences(QDialog):
             (u"Esperanto", "eo"),
             (u"Français", "fr"),
             (u"Italiano", "it"),
+            (u"Latviešu Valoda", "lv"),
             (u"Magyar", "hu"),
             (u"Nederlands","nl"),
             (u"Norsk","nb"),
@@ -53,6 +55,7 @@ class Preferences(QDialog):
             (u"Български", "bg"),
             (u"Монгол хэл","mn"),
             (u"русский язык", "ru"),
+            (u"українська мова", "uk"),
             (u"עִבְרִית", "he"),
             (u"العربية", "ar"),
             (u"فارسی", "fa"),
@@ -67,17 +70,22 @@ class Preferences(QDialog):
         self.setupNetwork()
         self.setupSave()
         self.setupAdvanced()
+        self.setupMedia()
         self.show()
 
     def accept(self):
         self.updateNetwork()
         self.updateSave()
         self.updateAdvanced()
+        self.updateMedia()
         self.config['interfaceLang'] = self.origConfig['interfaceLang']
         self.origConfig.update(self.config)
         self.origConfig.save()
         self.parent.setLang()
-        self.parent.moveToState("auto")
+        if self.needDeckClose:
+            self.parent.saveAndClose(parent=self)
+        else:
+            self.parent.reset()
         self.done(0)
 
     def reject(self):
@@ -98,11 +106,41 @@ class Preferences(QDialog):
         self.parent.setLang()
         self.dialog.retranslateUi(self)
 
+    def setupMedia(self):
+        self.dialog.mediaChoice.addItems(
+            QStringList([
+            _("Keep media next to deck"),
+            _("Keep media in DropBox"),
+            _("Keep media in custom folder"),
+            ]))
+        if not self.config['mediaLocation']:
+            idx = 0
+        elif self.config['mediaLocation'] == "dropbox":
+            idx = 1
+        else:
+            idx = 2
+        self.dialog.mediaChoice.setCurrentIndex(idx)
+        self.mediaChoiceChanged(idx)
+        self.connect(self.dialog.mediaChoice,
+                     SIGNAL("currentIndexChanged(int)"),
+                     self.mediaChoiceChanged)
+        self.origMediaChoice = idx
+
+    def mediaChoiceChanged(self, idx):
+        mp = self.dialog.mediaPath
+        mpl = self.dialog.mediaPrefix
+        if idx == 2:
+            mp.setText(self.config['mediaLocation'])
+            mp.setShown(True)
+            mpl.setShown(True)
+        else:
+            mp.setShown(False)
+            mpl.setShown(False)
+
     def setupNetwork(self):
         self.dialog.syncOnOpen.setChecked(self.config['syncOnLoad'])
-        self.dialog.syncOnClose.setChecked(self.config['syncOnClose'])
         self.dialog.syncOnProgramOpen.setChecked(self.config['syncOnProgramOpen'])
-        self.dialog.syncOnProgramClose.setChecked(self.config['syncOnProgramClose'])
+        self.dialog.disableWhenMoved.setChecked(self.config['syncDisableWhenMoved'])
         self.dialog.syncUser.setText(self.config['syncUsername'])
         self.dialog.syncPass.setText(self.config['syncPassword'])
         self.dialog.proxyHost.setText(self.config['proxyHost'])
@@ -114,9 +152,8 @@ class Preferences(QDialog):
 
     def updateNetwork(self):
         self.config['syncOnLoad'] = self.dialog.syncOnOpen.isChecked()
-        self.config['syncOnClose'] = self.dialog.syncOnClose.isChecked()
         self.config['syncOnProgramOpen'] = self.dialog.syncOnProgramOpen.isChecked()
-        self.config['syncOnProgramClose'] = self.dialog.syncOnProgramClose.isChecked()
+        self.config['syncDisableWhenMoved'] = self.dialog.disableWhenMoved.isChecked()
         self.config['syncUsername'] = unicode(self.dialog.syncUser.text())
         self.config['syncPassword'] = unicode(self.dialog.syncPass.text())
         self.config['proxyHost'] = unicode(self.dialog.proxyHost.text())
@@ -138,12 +175,28 @@ class Preferences(QDialog):
     def onOpenBackup(self):
         path = os.path.join(self.config.configPath, "backups")
         if sys.platform == "win32":
-            anki.latex.call(["explorer", path.encode(
+            anki.utils.call(["explorer", path.encode(
                 sys.getfilesystemencoding())],
                             wait=False)
         else:
             QDesktopServices.openUrl(QUrl("file://" + path))
 
+    def updateMedia(self):
+        orig = self.origMediaChoice
+        new = self.dialog.mediaChoice.currentIndex()
+        if orig == new and orig != 2:
+            return
+        if new == 0:
+            p = ""
+        elif new == 1:
+            p = "dropbox"
+            # reset public folder location
+            self.config['dropboxPublicFolder'] = ""
+        else:
+            p = unicode(self.dialog.mediaPath.text())
+        self.config['mediaLocation'] = p
+        self.needDeckClose = True
+
     def updateSave(self):
         self.config['saveAfterAnswer'] = self.dialog.saveAfterEvery.isChecked()
         self.config['saveAfterAnswerNum'] = self.dialog.saveAfterEveryNum.value()
@@ -166,7 +219,10 @@ class Preferences(QDialog):
         self.dialog.openLastDeck.setChecked(self.config['loadLastDeck'])
         self.dialog.deckBrowserOrder.setChecked(self.config['deckBrowserOrder'])
         self.dialog.deleteMedia.setChecked(self.config['deleteMedia'])
+        self.dialog.stripHTML.setChecked(self.config['stripHTML'])
+        self.dialog.autoplaySounds.setChecked(self.config['autoplaySounds'])
         self.dialog.deckBrowserLen.setValue(self.config['deckBrowserNameLength'])
+        self.dialog.optimizeSmall.setChecked(self.config['optimizeSmall'])
 
     def updateAdvanced(self):
         self.config['colourTimes'] = self.dialog.colourTimes.isChecked()
@@ -180,7 +236,10 @@ class Preferences(QDialog):
         self.config['alternativeTheme'] = self.dialog.alternativeTheme.isChecked()
         self.config['showProgress'] = self.dialog.showProgress.isChecked()
         self.config['preventEditUntilAnswer'] = self.dialog.preventEdits.isChecked()
+        self.config['stripHTML'] = self.dialog.stripHTML.isChecked()
+        self.config['autoplaySounds'] = self.dialog.autoplaySounds.isChecked()
         self.config['loadLastDeck'] = self.dialog.openLastDeck.isChecked()
+        self.config['optimizeSmall'] = self.dialog.optimizeSmall.isChecked()
         if self.dialog.deckBrowserOrder.isChecked():
             self.config['deckBrowserOrder'] = 1
         else:
