@@ -1,4 +1,4 @@
 #!/bin/bash
 
 rsync -av --delete $c/libanki $c/ankiqt mari:anki/
-ssh mari 'cd anki && ./make.sh && scp Anki.dmg reflex:'
+ssh mari 'cd anki && ./make.sh && scp Anki.dmg twitch:'
